﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections.ObjectModel;
using System.Data.EntityClient;
using System.Data.SqlClient;
using DeepViewDataModel.DataModel;
using DeepViewDataModel.Common;
using System.Collections.Concurrent;
using System.IO;
using DZConverterLib.DAL;
using DZConverterLib.Auth;
using pdftron;
using System.Data.Objects.DataClasses;
using pdftron.PDF;
using System.Drawing;
using System.Drawing.Imaging;
using pdftron.SDF;
using Microsoft.DeepZoomTools;
using DeepView.Toolkit.GuidGenerator;
using System.Threading;
using System.Net.Cache;
namespace DZConverterLib
{
    public class DZConverter: IDisposable
    {
        //public const string ProjectRoot = @"D:\DeepViewConverter\";
        //public const string DownloadDir_PDF = ProjectRoot + @"\PDFPath";
        public string DownloadDir_PDF;
        private List<String> DeepZoopDest = null;
        private EntityCollection<DocPage> pageSet = null;
        public int pageCount = 0;
        public ConverterPDF ConverterItem;
        public event EventHandler<MakeConverterErrorArgs> ConverterMakeError;
        public event EventHandler<EndConverterEventArgs> EndConverter;

        public DZConverter()
        {
            pdftron.PDFNet.Initialize("daelim.co.kr(Doftech Corp):CPU:2::W:AMC(20120315):EF6E886F25A414FFB5F8C1F2999CF2DA33DC6C5164315BAF7011B87AF0FA");
            DeepZoopDest = new List<string>();
            pageSet = new EntityCollection<DocPage>();
        }

        private void SendError(string Error,bool ThreadStop)
        {
            if (ConverterMakeError != null)
                ConverterMakeError(this, new MakeConverterErrorArgs { ThreadStop = ThreadStop, ConverterID = ConverterItem.ID, Message = Error });
        }

        public void GetSilgleFile(object obj)
        {
            try
            {
                ConverterItem = obj as ConverterPDF;

                using (DeepViewEntities _deepview = new DeepViewEntities(ConnectStringBuilder.DeepViewConnectionString().ToString()))
                {
                    var _property = _deepview.Properties.Where(pro => pro.Property == ConverterItem.ProjectNo);

                    if (_property.Count() > 0)
                    {
                        DownloadDir_PDF = _property.Where(t => t.Type == PropertiesType.PropertiesType.Const_TileSorceStorage).First().Value;

                        if (!Directory.Exists(DownloadDir_PDF)) // 폴더 없을 시 생성
                        {
                            Directory.CreateDirectory(DownloadDir_PDF);
                        }
                    }
                }

                    ConvertProcess();
               
            }
            catch (Exception e)
            {
                SendError("GetSilgleFile : " + e.ToString(), true);
            }
        }

        private void SetState(IConverterPDF.ConverterStatus Status)
        {
            using (DeepViewEntities _entity = new DeepViewEntities(ConnectStringBuilder.DeepViewConnectionString().ToString()))
            {
                var _items = _entity.ConverterPDF.Where(converter => converter.ID == this.ConverterItem.ID);
                if (_items.Count() > 0)
                {
                    _items.First().Status = (int)Status;
                    _entity.SaveChanges();
                }
            }  
        }

        private void SetCurrentPage(int CurrentPage)
        {
            using (DeepViewEntities _entity = new DeepViewEntities(ConnectStringBuilder.DeepViewConnectionString().ToString()))
            {
                var _items = _entity.ConverterPDF.Where(converter => converter.ID == this.ConverterItem.ID);
                if (_items.Count() > 0)
                {
                    _items.First().CurrentPage = CurrentPage;
                    _entity.SaveChanges();
                }
            }
        }

        private static object _lock = new object();

        public static void DeleteDirectory(string target_dir)
        {
            DeleteDirectoryFiles(target_dir);
            while (Directory.Exists(target_dir))
            {
                lock (_lock)
                {
                    DeleteDirectoryDirs(target_dir);
                }
            }
        }

        private static void DeleteDirectoryDirs(string target_dir)
        {
            System.Threading.Thread.Sleep(100);

            if (Directory.Exists(target_dir))
            {

                string[] dirs = Directory.GetDirectories(target_dir);

                if (dirs.Length == 0)
                    Directory.Delete(target_dir, true);
                else
                    foreach (string dir in dirs)
                        DeleteDirectoryDirs(dir);
            }
        }

        private static void DeleteDirectoryFiles(string target_dir)
        {
            string[] files = Directory.GetFiles(target_dir);
            string[] dirs = Directory.GetDirectories(target_dir);

            foreach (string file in files)
            {
                File.SetAttributes(file, FileAttributes.Normal);
                File.Delete(file);
            }

            foreach (string dir in dirs)
            {
                DeleteDirectoryFiles(dir);
            }
        }

        void DeteteDirectory(string Path)
        {
            Thread _stathread = new Thread(new ParameterizedThreadStart(DeteteWork));
            _stathread.SetApartmentState(ApartmentState.MTA);
            _stathread.Start(Path);
        }

        void DeteteWork(object Path)
        {
            DirectoryInfo _dir = new DirectoryInfo(Path.ToString());

            _dir.EnumerateDirectories().AsParallel().ForAll(d =>
            {
                d.Delete(true);
            });

            _dir.Delete(true);
        }

        public void ConvertProcess()
        {
            #region 기본 설정
            string DocUri = ConverterItem.PdfUrl; //PDF 전체 경로
            string FileName = DocUri.Remove(0, DocUri.LastIndexOf("/") + 1);
            string ProjectFolderPath = DownloadDir_PDF + "\\" + ConverterItem.ProjectNo + "_Tile"; //프로젝트 폴더
            string ItemListPath = ProjectFolderPath + "\\" + (System.Convert.ToInt64(ConverterItem.SharepointItemID)/100).ToString();
            string ItemPath = ItemListPath + "\\" + ConverterItem.SharepointItemID;
            string DownloadFilePath = ItemPath + "\\" + FileName;
            string _delPath = ItemPath;
            #endregion

            #region 폴더 생성

            try
            {
                DirectoryInfo _dir = new DirectoryInfo(ItemPath);

                if (_dir.Exists)
                {
                    for (int i = 0; i < 100; i++)
                    {
                        _delPath = ItemPath + "_" + i.ToString();
                        if (!Directory.Exists(ItemPath + "_" + i.ToString()))
                        {
                            _dir.MoveTo(_delPath);
                            DeteteDirectory(_delPath);
                            break;
                        }
                    }
                }

                Directory.CreateDirectory(ItemPath);

            }
            catch (Exception ex)
            {
                SendError(" 폴더삭제 에러 " + ex.ToString(), false);
            }
  
            #endregion

            #region 다운로드

            #region 다운로드
            Filefactory _fileFactory = new Filefactory();
            if (AuthUser2(_fileFactory, DocUri))
            {
                _fileFactory.DownloadFile(new Uri(DocUri), DownloadFilePath);
            }
            else
            {
                SendError("다운로드에 실패하였습니다", true);
            }
            #endregion      
            //System.Net.WebClient _fileFactory = new System.Net.WebClient();
            ////HttpRequestCachePolicy policy = new HttpRequestCachePolicy(HttpRequestCacheLevel.CacheIfAvailable);
            ////_fileFactory.CachePolicy = policy;
            //var _auth = AuthUser(DocUri);
            //if (_auth != null)
            //{
            //    _fileFactory.Credentials = _auth;
            //    _fileFactory.DownloadFile(new Uri(DocUri), DownloadFilePath);
            //}
            //else
            //{
            //    SendError("다운로드에 실패하였습니다", true);
            //} 
                #endregion           

            ResultSet Result_Check;
            #region PDF 체크
            try
            {
                Result_Check = Check_PDF(DownloadFilePath);
            }
            catch (Exception ex)
            {
                System.Diagnostics.Debug.WriteLine(ex.ToString());
                throw;
            }
            
            if (!Result_Check.result)
            {
                SendError(" PDF 체크 실패, 원인 : " + Result_Check.Message, true);
            } 
            #endregion

            SetState(IConverterPDF.ConverterStatus.Create);

            #region PDF에서 이미지로 변환
            if (!ConvertImage(ItemPath, FileName))
            {
                SendError("PDF 이미지로 변환 실패, 원인 : 데이터베이스 쓰기 실패", true);
            } 
            #endregion

            SetState(IConverterPDF.ConverterStatus.Crop);

            #region 이미지에서 딥줌 이미지로
            if (!ConvertDeepZoomImage(DeepZoopDest))
            {
                SendError("딥줌 이미지 변경 실패", true);
            }
            #endregion

            #region 데이터베이스 입력
            if (!SetPageInfoToDB())
            {
                SendError("데이터베이스 입력 실패", true);
            }
            #endregion

            //// virtual dir생성은 로컬에서만 가능
            //try
            //{
            //    VirtualDirCreate.CreateVirtualDir("comment.daelim.com", "MACHINE/WEBROOT/APPHOST/DeepView", prjNo + "_Tile", DownloadDir_PDF + "\\" + prjNo + "_Tile");
            //}
            //catch (Exception ex)
            //{
            //    SendError("VirtualDirCreate Error");
            //}

            if (EndConverter != null)
                EndConverter(this, new EndConverterEventArgs { ConverterPDF = ConverterItem});
        }

        public ResultSet Check_PDF(string filePath)
        {
            pdftron.PDF.PDFDoc doc = new pdftron.PDF.PDFDoc(filePath);
            #region 접근권한 체크
            if (!doc.InitSecurityHandler())
            {
                PDFNet.Terminate();
                return new ResultSet { result = false, Message = "Document authentication Error" };
            } 
            #endregion
            #region 암호화 여부
            if (doc.IsEncrypted())
            {
                PDFNet.Terminate();
                return new ResultSet { result = false, Message = "Document is Encrypted" };
            } 
            #endregion            
            return new ResultSet { result = true };
        }


        #region private Method
        private bool AuthUser2(Filefactory fileFactory, string PDFurl)
        {
            try
            {
                System.Net.HttpWebRequest request = (System.Net.HttpWebRequest)System.Net.WebRequest.Create(PDFurl);
                System.Net.NetworkCredential credentials = Crediential.GetAdminCrediential(PDFurl.ToString());
                request.Credentials = credentials;
                System.Net.HttpWebResponse response = (System.Net.HttpWebResponse)request.GetResponse();
                response.Close();
                fileFactory.Credentials = credentials;
            }
            catch (Exception ex)
            {
                System.Diagnostics.Debug.WriteLine(ex.ToString());
                return false;
            }

            return true;
        }
        private System.Net.ICredentials AuthUser(string PDFurl)
        {
            System.Net.NetworkCredential credentials = null;
            try
            {
                var _url = new UriBuilder(new Uri(PDFurl).AbsoluteUri.Replace(new Uri(PDFurl).PathAndQuery, "/"));
                var _dns = _url.Host.Split('.');

                if (_dns.Count() == 3)
                    if (_dns[0].ToLower() != "www")
                    {
                        _dns[0] = "www";
                        _url.Host = _dns[0] + "." + _dns[1] + "." + _dns[2];
                    }

                System.Net.HttpWebRequest request = (System.Net.HttpWebRequest)System.Net.WebRequest.Create(_url.Uri);
                credentials = new System.Net.NetworkCredential("DAELIM\\esbsystem", "kdm707");
                request.Credentials = credentials;
                System.Net.HttpWebResponse response = (System.Net.HttpWebResponse)request.GetResponse();   
                response.Close();
            }
            catch (Exception ex)
            {
                return credentials;
            }

            return credentials;
        }

        private ImageCodecInfo GetEncoderInfo(String mimeType)
        {
            int j;
            ImageCodecInfo[] encoders;
            encoders = ImageCodecInfo.GetImageEncoders();
            for (j = 0; j < encoders.Length; ++j)
            {
                if (encoders[j].MimeType == mimeType)
                    return encoders[j];
            }
            return null;
        }
        private bool ConvertImage(string FilePath, string fileName)
        {
            pageSet = new EntityCollection<DocPage>();
            using (PDFDoc doc = new PDFDoc(FilePath + "\\" + fileName))
            {
                pdftron.PDFNet.Initialize("daelim.co.kr(Doftech Corp):CPU:2::W:AMC(20120315):EF6E886F25A414FFB5F8C1F2999CF2DA33DC6C5164315BAF7011B87AF0FA");
                Bitmap newBmp_;
                ImageCodecInfo DefaultImageCodecInfo = GetEncoderInfo("image/png");
                EncoderParameters DefaultEncoderParameters = new EncoderParameters(2);
                System.Drawing.Imaging.Encoder QualityEncoder = System.Drawing.Imaging.Encoder.Quality;
                System.Drawing.Imaging.Encoder ColorDepthEncoder = System.Drawing.Imaging.Encoder.ColorDepth;
                DefaultEncoderParameters.Param[0] = new EncoderParameter(QualityEncoder, 100L);
                DefaultEncoderParameters.Param[1] = new EncoderParameter(ColorDepthEncoder, 8L);
                DeepZoopDest = new List<string>();
                pageCount = doc.GetPageCount();

                using (DeepViewEntities _entity = new DeepViewEntities(ConnectStringBuilder.DeepViewConnectionString().ToString()))
                {
                    var _items = _entity.ConverterPDF.Where(converter => converter.ID == this.ConverterItem.ID);
                    if (_items.Count() > 0)
                    {
                        _items.First().TotalPages = pageCount;
                        _entity.SaveChanges();
                    }
                }

                #region 이미지 만들기
                for (int i = 1; i < doc.GetPageCount() + 1; i++)
                {
                    try
                    {
                        SetCurrentPage(i);
                        using (pdftron.PDF.PDFDraw draw = new pdftron.PDF.PDFDraw())
                        {
                            ElementBuilder bld = new ElementBuilder();
                            ElementWriter writer = new ElementWriter();

                            var widthData = (int)doc.GetPage(i).GetPageWidth();
                            int heightData = (int)doc.GetPage(i).GetPageHeight();
                            pageSet.Add(new DocPage 
                                        {
                                            Id=  GuidGenerator.GetUniqueGuid() , 
                                            PageWidth = (widthData*2).ToString(), 
                                            PageHeight = (heightData*2).ToString(), 
                                            PageNumber = i, 
                                            PageAngle = 0,
                                        });

                            var rotation = doc.GetPage(i).GetRotation();
                            draw.SetImageSize((int)(widthData * 2), (int)(heightData * 2), true);
                            //draw.SetImageSize((int)(widthData), (int)(heightData), true);
                            draw.SetAntiAliasing(true);
                            draw.SetImageSmoothing(true);
                            newBmp_ = draw.GetBitmap(doc.GetPage(i));

                            using (MemoryStream _savestream = new MemoryStream())
                            {

                                //newBmp_.Save(@"c:\test.jpg");
                                newBmp_.Save(_savestream, DefaultImageCodecInfo, DefaultEncoderParameters);
                                //newBmp_.Save(_savestream, ImageFormat.Png);
                                newBmp_ = new Bitmap(_savestream);
                                ObjSet objset = new ObjSet();
                                Obj jbig2_hint = objset.CreateName("png");

                                string pagePath = FilePath + "\\" + i;

                                Directory.CreateDirectory(pagePath);
                                DeepZoopDest.Add(pagePath + "\\" + i + ".jpg");
                                newBmp_.Save(pagePath + "\\" + i + ".jpg");
                                newBmp_.Dispose();
      
                            }

                            GC.Collect();
                            GC.WaitForPendingFinalizers();
                        }
                    }
                    catch (Exception ex)
                    {
                        SendError(ex.ToString(), false);
                        return false;
                    }
                }
                #endregion
                return true;
            }
        }

        private bool ConvertDeepZoomImage(List<String> FileSet)
        {
            ImageCreator crator = new ImageCreator();
            crator.TileSize = 256;
            crator.TileFormat = Microsoft.DeepZoomTools.ImageFormat.AutoSelect;
            crator.CopyMetadata = false;

            try
            {
                for (int i = 0; i < FileSet.Count; i++)
                {
                        SetCurrentPage(i + 1);
                        crator.Create(FileSet[i], FileSet[i].ToString().Replace(".jpg", ".xml"));
                }
            }
            catch (Exception ex)
            {
                System.Diagnostics.Debug.WriteLine(ex.ToString());
                new Exception("Converter Image Error");
            }
              
            return true;
        }

        private bool SetPageInfoToDB()
        {
            #region 다 쓴 이미지 삭제
            foreach (var item in DeepZoopDest)
            {
                try
                {
                    if (File.Exists(item.ToString()))
                    {
                        File.Delete(item.ToString());
                    }
                }
                catch (Exception e)
                {
                    /// 서버로 보내는 에러메시지 필요
                }

            } 
            #endregion
            #region DB에 페이지 정보 입력
            try
            {

                using (CI_Entities deepViewEntity = new CI_Entities(ConnectStringBuilder.ProjectCIConnectString(ConverterItem.ProjectNo).ToString()))
                {
                    var _docinfo = deepViewEntity
                                    .DocInfo.Where(info => info.OrginPDFUrl == ConverterItem.PdfUrl);

                    if (_docinfo.Count() > 0)
                    {
                        System.Diagnostics.Stopwatch _watch = new System.Diagnostics.Stopwatch();
                        _watch.Start();
                        var _doc = _docinfo.First();
                        _doc.SharepointItemID = ConverterItem.SharepointItemID;
                        _doc.PageCount = pageCount;

                        pageSet.AsParallel().ForAll(p => p.DocInfo_Id = _doc.Id);

                        var _exceptDocInfo = (from newinfo in pageSet
                                              select new { newinfo.PageNumber, newinfo.DocInfo_Id,newinfo.PageWidth,newinfo.PageHeight})
                                                  .Except
                                                        (from oldinfo in _doc.DocPage
                                                         select new { oldinfo.PageNumber, oldinfo.DocInfo_Id, oldinfo.PageWidth, oldinfo.PageHeight });

                        var _delDocInfo = (from oldinfo in _doc.DocPage
                                           select new { oldinfo.PageNumber, oldinfo.DocInfo_Id, oldinfo.PageWidth, oldinfo.PageHeight })
                                                  .Except
                                                        (from newinfo in pageSet
                                                         select new { newinfo.PageNumber, newinfo.DocInfo_Id, newinfo.PageWidth, newinfo.PageHeight })
                                                         .ToList(); /// 삭제되는 collection은 하나씩 제거 될때마다 수정되기 때문에
                                                                    ///  foreach가 안된다. 그러므로 tolist로 가져와서 작업한다.

                        foreach (var item in _delDocInfo)
                        {
                            var _delitem = _doc.DocPage.Where(p => p.PageNumber == item.PageNumber).First();
                             deepViewEntity.DocPage.DeleteObject(_delitem);
                            //deepViewEntity.DocPage.DeleteObject(item);
                        }
                        
                        deepViewEntity.SaveChanges();

                        foreach (var exceptitem in _exceptDocInfo)
                        {
                            var _lstPage = deepViewEntity.DocPage.Where(page => page.PageNumber == exceptitem.PageNumber
                                                                                    && page.DocInfo_Id == exceptitem.DocInfo_Id);

                            if (_lstPage.Count() > 0)
                            {
                                var _page = _lstPage.First();
                                _page.PageHeight = exceptitem.PageHeight;
                                _page.PageWidth = exceptitem.PageWidth;
                            }
                            else
                            {
                                _doc.DocPage.Add(pageSet.Where(p=>p.PageNumber == exceptitem.PageNumber).First());
                            }
                        }

                        deepViewEntity.SaveChanges();

                        _watch.Stop();
                        System.Diagnostics.Debug.WriteLine(_watch.ElapsedMilliseconds);
                    }
                    else
                    {

                        Guid guid = GuidGenerator.GetUniqueGuid();
                        DocInfo instace = new DocInfo
                        {
                            Id = guid,
                            OrginPDFUrl = ConverterItem.PdfUrl,
                            SharepointItemID = ConverterItem.SharepointItemID,
                            PageCount = pageCount,
                            DocPage = pageSet
                        };
                        deepViewEntity.DocInfo.AddObject(instace);
                    }

                    deepViewEntity.SaveChanges();
                }
            }
            catch (Exception e)
            {
                SendError("SetPageInfoToDB : " + e.ToString(),true);
                return false;
            }
               
            #endregion

            return true;
        }
    #endregion

        public void Dispose()
        {
            pdftron.PDFNet.Terminate();
            GC.Collect();
            GC.SuppressFinalize(this);
        }
     
    }

    public enum FinalStatus
    {
        /// <summary>
        /// Final PDF를 만들기 위한 순서에 추가
        /// </summary>
        Insert = 0,

        /// <summary>
        /// 앞의 final이 처리중일때 대기
        /// </summary>
        Wait = 1,

        /// <summary>
        /// 만들고 있는중
        /// 이때 데이터베이스에 Current Page는 만들고 있는 페이지
        /// </summary>
        Create = 2,

        /// <summary>
        /// final pdf 완료시
        /// </summary>
        Seccess = 99,

        /// <summary>
        /// 에러
        /// </summary>
        Error = 88
    }

}
